==> python-beartype_0.20.2.orig.tar.gz: beartype-0.20.2/beartype/_check/code/_pep/pep484/codepep484typevar.py <==
#!/usr/bin/env python3
# --------------------( LICENSE                            )--------------------
# Copyright (c) 2014-2025 Beartype authors.
# See "LICENSE" for further details.

'''
Beartype :pep:`484`-compliant **type variable type-checking code factories**
(i.e., low-level callables dynamically generating pure-Python code snippets
type-checking arbitrary objects against type variables).

This private submodule is *not* intended for importation by downstream callers.
'''

#FIXME: Excise us up but preserve this submodule, please. We'll want this once
#we begin generating call-time code type-checking type variables.

# # ....................{ IMPORTS                            }....................
# from beartype.typing import Optional
# from beartype._check.code.codecls import HintMeta
# from beartype._check.convert.convsanify import (
#     sanify_hint_child)
# from beartype._conf.confcls import BeartypeConf
# from beartype._data.hint.datahintpep import Hint
# from beartype._data.hint.datahinttyping import TypeStack
# from beartype._util.hint.pep.proposal.pep484.pep484typevar import (
#     get_hint_pep484_typevar_bound_or_none)
#
# # ....................{ FACTORIES                          }....................
# def make_hint_pep484_typevar_check_expr(
#     # Mandatory parameters.
#     hint_meta: HintMeta,
#     conf: BeartypeConf,
#     pith_curr_assign_expr: str,
#     pith_curr_var_name_index: int,
#
#     # Optional parameters.
#     cls_stack: TypeStack = None,
#     exception_prefix: str = '',
# ) -> Optional[Hint]:
#     '''
#     Either reduce the mostly semantically useless passed :pep:`484`-compliant
#     **type variable** (i.e., :class:`typing.TypeVar` object) to the semantically
#     useful bounds or constraints of this variable if any *or* silently ignore
#     this variable otherwise (i.e., if this variable is neither bounded nor
#     constrained)..
#
#     This factory is intentionally *not* memoized (e.g., by the
#     :func:`.callable_cached` decorator), due to accepting **context-sensitive
#     parameters** (i.e., whose values contextually depend on context unique to
#     the code being generated for the currently decorated callable) such as
#     ``pith_curr_assign_expr``.
#
#     Parameters
#     ----------
#     hint_meta : HintMeta
#         Metadata describing the currently visited hint, appended by the
#         previously visited parent hint to the ``hints_meta`` stack.
#     conf : BeartypeConf
#         **Beartype configuration** (i.e., self-caching dataclass encapsulating
#         all settings configuring type-checking for the passed object).
#     pith_curr_assign_expr : str
#         Assignment expression assigning this full Python expression to the
#         unique local variable assigned the value of this expression.
#     pith_curr_var_name_index : int
#         Integer suffixing the name of each local variable assigned the value of
#         the current pith in a assignment expression, thus uniquifying this
#         variable in the body of the current wrapper function.
#     cls_stack : TypeStack, optional
#         **Type stack** (i.e., either a tuple of the one or more
#         :func:`beartype.beartype`-decorated classes lexically containing the
#         class variable or method annotated by this hint *or* :data:`None`).
#         Defaults to :data:`None`.
#     exception_prefix : str, optional
#         Human-readable substring prefixing the representation of this object in
#         the exception message. Defaults to the empty string.
#
#     Returns
#     -------
#     Optional[Hint]
#         Either:
#
#         * If this type variable reduces to an unignorable type hint, that hint.
#         * Else, :data:`None` (i.e., if this type variable is ignorable).
#     '''
#     assert isinstance(hint_meta, HintMeta), (
#         f'{repr(hint_meta)} not "HintMeta" object.')
#
#     # ....................{ LOCALS                         }....................
#     # This type variable, localized for negligible efficiency gains. *sigh*
#     hint = hint_meta.hint
#
#     # Hint mapped to by this type variable if one or more transitive parent
#     # hints previously mapped this type variable to a hint *OR* "None".
#     hint_child: Optional[Hint] = hint_meta.typevar_to_hint.get(hint)  # pyright: ignore
#
#     # ....................{ REDUCTION                      }....................
#     # If *NO* transitive parent hints previously mapped this type variable to a
#     # hint...
#     if hint_child is None:
#         # PEP-compliant hint synthesized from all bounded constraints
#         # parametrizing this type variable if any *OR* "None" otherwise.
#         #
#         # Note this call is intentionally passed positional rather positional
#         # keywords due to memoization.
#         hint_curr_bound = get_hint_pep484_typevar_bound_or_none(
#             hint, exception_prefix)  # pyright: ignore
#
#         # If this type variable was parametrized by one or more bounded
#         # constraints, reduce this type variable to these bounded constraints.
#         if hint_curr_bound is not None:
#             hint_child = hint_curr_bound
#         # Else, this type variable was unparametrized. In this case, preserve
#         # this type variable as is.
#     # Else, one or more transitive parent hints previously mapped this type
#     # variable to a hint.
#
#     # If this type variable was either mapped to another hint by one or more
#     # transitive parent hints *OR* parametrized by one or more bounded
#     # constraints...
#     if hint_child is not None:
#         # Unignorable sane hint sanified from this possibly ignorable insane
#         # hint *OR* "None" otherwise (i.e., if this hint is ignorable).
#         hint_child = sanify_hint_child(
#             hint=hint_child,
#             conf=conf,
#             cls_stack=cls_stack,
#             exception_prefix=exception_prefix,
#         )
#     # Else, this type variable was neither mapped to another hint by one or more
#     # transitive parent hints *NOR* parametrized by one or more bounded
#     # constraints.
#
#     # If this type variable is reducible to an unignorable hint...
#     if hint_child is not None:
#         # Ignore this semantically useless type variable in favour of this
#         # semantically useful hint by replacing *ALL* hint metadata describing
#         # the former with the latter.
#         hint_meta.reinit(
#             hint=hint_child,  # pyright: ignore
#             indent_level=hint_meta.indent_level + 1,
#             pith_expr=pith_curr_assign_expr,
#             pith_var_name_index=pith_curr_var_name_index,
#             typevar_to_hint=hint_meta.typevar_to_hint,
#         )
#     # Else, this type variable is *NOT* reducible to an unignorable hint. Since
#     # @beartype currently fails to generate type-checking code for type
#     # variables in and of themselves, type variables have *NO* intrinsic
#     # semantic meaning and are thus ignorable.
#
#     # ....................{ RETURN                         }....................
#     # Return either the unignorable hint this type variable reduces to if any
#     # *OR* "None" otherwise (i.e., if this type variable is ignorable).
#     return hint_child

==> python-beartype_0.22.9.orig.tar.gz: beartype-0.22.9/beartype/_check/code/_pep/pep484/codepep484typevar.py <==
#!/usr/bin/env python3
# --------------------( LICENSE                            )--------------------
# Copyright (c) 2014-2025 Beartype authors.
# See "LICENSE" for further details.

'''
Beartype :pep:`484`-compliant **type variable type-checking code factories**
(i.e., low-level callables dynamically generating pure-Python code snippets
type-checking arbitrary objects against type variables).

This private submodule is *not* intended for importation by downstream callers.
'''

#FIXME: Excise us up but preserve this submodule, please. We'll want this once
#we begin generating call-time code type-checking type variables.

# # ....................{ IMPORTS                            }....................
# from beartype.typing import Optional
# from beartype._check.code.codecls import HintMeta
# from beartype._check.convert.convmain import (
#     sanify_hint_child)
# from beartype._conf.confmain import BeartypeConf
# from beartype._data.typing.datatypingport import Hint
# from beartype._data.typing.datatyping import TypeStack
# from beartype._util.hint.pep.proposal.pep484.pep484typevar import (
#     get_hint_pep484_typevar_bounded_constraints_or_none)
#
# # ....................{ FACTORIES                          }....................
# def make_hint_pep484_typevar_check_expr(
#     # Mandatory parameters.
#     hint_meta: HintMeta,
#     conf: BeartypeConf,
#     pith_curr_assign_expr: str,
#     pith_curr_var_name_index: int,
#
#     # Optional parameters.
#     cls_stack: TypeStack = None,
#     exception_prefix: str = '',
# ) -> Optional[Hint]:
#     '''
#     Either reduce the mostly semantically useless passed :pep:`484`-compliant
#     **type variable** (i.e., :class:`typing.TypeVar` object) to the semantically
#     useful bounds or constraints of this variable if any *or* silently ignore
#     this variable otherwise (i.e., if this variable is neither bounded nor
#     constrained)..
#
#     This factory is intentionally *not* memoized (e.g., by the
#     :func:`.callable_cached` decorator), due to accepting **context-sensitive
#     parameters** (i.e., whose values contextually depend on context unique to
#     the code being generated for the currently decorated callable) such as
#     ``pith_curr_assign_expr``.
#
#     Parameters
#     ----------
#     hint_meta : HintMeta
#         Metadata describing the currently visited hint, appended by the
#         previously visited parent hint to the ``hints_meta`` stack.
#     conf : BeartypeConf
#         **Beartype configuration** (i.e., self-caching dataclass encapsulating
#         all settings configuring type-checking for the passed object).
#     pith_curr_assign_expr : str
#         Assignment expression assigning this full Python expression to the
#         unique local variable assigned the value of this expression.
#     pith_curr_var_name_index : int
#         Integer suffixing the name of each local variable assigned the value of
#         the current pith in a assignment expression, thus uniquifying this
#         variable in the body of the current wrapper function.
#     cls_stack : TypeStack, optional
#         **Type stack** (i.e., either a tuple of the one or more
#         :func:`beartype.beartype`-decorated classes lexically containing the
#         class variable or method annotated by this hint *or* :data:`None`).
#         Defaults to :data:`None`.
#     exception_prefix : str, optional
#         Human-readable substring prefixing the representation of this object in
#         the exception message. Defaults to the empty string.
#
#     Returns
#     -------
#     Optional[Hint]
#         Either:
#
#         * If this type variable reduces to an unignorable type hint, that hint.
#         * Else, :data:`None` (i.e., if this type variable is ignorable).
#     '''
#     assert isinstance(hint_meta, HintMeta), (
#         f'{repr(hint_meta)} not "HintMeta" object.')
#
#     # ....................{ LOCALS                         }....................
#     # This type variable, localized for negligible efficiency gains. *sigh*
#     hint = hint_meta.hint
#
#     # Hint mapped to by this type variable if one or more transitive parent
#     # hints previously mapped this type variable to a hint *OR* "None".
#     hint_child: Optional[Hint] = hint_meta.typearg_to_hint.get(hint)  # pyright: ignore
#
#     # ....................{ REDUCTION                      }....................
#     # If *NO* transitive parent hints previously mapped this type variable to a
#     # hint...
#     if hint_child is None:
#         # PEP-compliant hint synthesized from all bounded constraints
#         # parametrizing this type variable if any *OR* "None" otherwise.
#         #
#         # Note this call is intentionally passed positional rather positional
#         # keywords due to memoization.
#         hint_curr_bound = get_hint_pep484_typevar_bounded_constraints_or_none(
#             hint, exception_prefix)  # pyright: ignore
#
#         # If this type variable was parametrized by one or more bounded
#         # constraints, reduce this type variable to these bounded constraints.
#         if hint_curr_bound is not None:
#             hint_child = hint_curr_bound
#         # Else, this type variable was unparametrized. In this case, preserve
#         # this type variable as is.
#     # Else, one or more transitive parent hints previously mapped this type
#     # variable to a hint.
#
#     # If this type variable was either mapped to another hint by one or more
#     # transitive parent hints *OR* parametrized by one or more bounded
#     # constraints...
#     if hint_child is not None:
#         # Unignorable sane hint sanified from this possibly ignorable insane
#         # hint *OR* "None" otherwise (i.e., if this hint is ignorable).
#         hint_child = sanify_hint_child(
#             hint=hint_child,
#             conf=conf,
#             cls_stack=cls_stack,
#             exception_prefix=exception_prefix,
#         )
#     # Else, this type variable was neither mapped to another hint by one or more
#     # transitive parent hints *NOR* parametrized by one or more bounded
#     # constraints.
#
#     # If this type variable is reducible to an unignorable hint...
#     if hint_child is not None:
#         # Ignore this semantically useless type variable in favour of this
#         # semantically useful hint by replacing *ALL* hint metadata describing
#         # the former with the latter.
#         hint_meta.reinit(
#             hint=hint_child,  # pyright: ignore
#             indent_level=hint_meta.indent_level + 1,
#             pith_expr=pith_curr_assign_expr,
#             pith_var_name_index=pith_curr_var_name_index,
#             typearg_to_hint=hint_meta.typearg_to_hint,
#         )
#     # Else, this type variable is *NOT* reducible to an unignorable hint. Since
#     # @beartype currently fails to generate type-checking code for type
#     # variables in and of themselves, type variables have *NO* intrinsic
#     # semantic meaning and are thus ignorable.
#
#     # ....................{ RETURN                         }....................
#     # Return either the unignorable hint this type variable reduces to if any
#     # *OR* "None" otherwise (i.e., if this type variable is ignorable).
#     return hint_child
